/*******************************************************************
*
*  DESCRIPTION: Buzzer Driver Atomic Model
*
*  This is the implementation of the buzzer driver atomic model.
*  Its function is to determine when the buzzer should be turned
*  on or off.  The model has five inputs and one output defined 
*  as follows:
*
*  <time_set>: This input is used by the driver so it knows not to 
*  activate the buzzer when time of day is being set manually.
*
*  <alarm_set>: This input is used by the driver so it knows not to 
*  activate the buzzer when the alarm time is being set manually.
*
*  <time_match>: This input tells the driver that a match occured 
*  between the time of day and the alarm time.
*
*  <alarm_on>: This input enables and disables the alarm.  When 
*  it is true the alarm is armed.  When false, the alarm is
*  de-armed which means the buzzer can never be sounded.
*
*  <snooze>: This input is used to turn the buzzer off for a period
*  of 10 minutes.
*
*  <buzzer_on>: When set to true this output indicates the buzzer 
*  is on.  When set to false it indicates the buzzer is off.
*
*  AUTHOR: Christian Jacques
*
*  EMAIL: chris.jacques@videotron.ca
*
*  DATE: 05 October 2001
*
*******************************************************************/

/** include files **/
#include "buzzerDriver.h"  // class BuzDrv 
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: BuzDrv constructor
* Description: This routine constructs the BuzDrv model
********************************************************************/
BuzDrv::BuzDrv( const string &name )
: Atomic( name )
, time_set( addInputPort( "time_set" ) )
, alarm_set( addInputPort( "alarm_set" ) )
, time_match( addInputPort( "time_match" ) )
, alarm_on( addInputPort( "alarm_on" ) )
, snooze( addInputPort( "snooze" ) )
, buzzer_on( addOutputPort( "buzzer_on" ) )
, one_hour( 1, 0, 0, 0 )
, ten_minutes( 0, 10, 0, 0 )
{
}

/*******************************************************************
* Function Name: BuzDrv init function
* Description: This routine is invoked when simulation starts.
* This method simply initializes some state variables.
********************************************************************/
Model &BuzDrv::initFunction()
{

	// initialize state variables
	this->timeRemaining = Time::Zero;
	this->alarmTime = Time::Zero;
	this->armed = false;
	this->alarmSet = false;
	this->timeSet = false;
	this->snooz = false;
	this->buzzer = false;

	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: This method handles external events coming from any
* of the five input ports.
********************************************************************/
Model &BuzDrv::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == this->alarm_on )
	   {		
	   if( msg.value() == true )
		{
		// arm the driver and wait for an external event
		this->armed = true;
		this->passivate();
		}
	   else
	   	{
		// de-arm the driver and reset some of the 
		// state variables.
		this->armed = false;
		this->buzzer = false;
		this->snooz = false;
		
		if( this->state() == active )
		   // turn off the buzzer
		   holdIn( passive, Time::Zero );
		else
		   // buzzer was not on.  Just wait for
		   // external events
		   this->passivate();
		}
	   }
	else if( msg.port() == this->time_set )
	   this->timeSet = msg.value();

	else if( msg.port() == this->alarm_set )
	   this->alarmSet = msg.value();

	else if( msg.port() == this->time_match )
	   {
	   // turn buzzer on if the right conditions are met
	   if( this->armed && !this->alarmSet && !this->timeSet )
		{
		this->alarmTime = msg.time();
		this->timeRemaining = one_hour;
		this->buzzer = true;
		this->snooz = false;
		holdIn( active, Time::Zero );
		}
	   }
	else if( msg.port() == this->snooze )
	   {
	   if( this->state() == active )
		{
		this->timeRemaining = alarmTime + one_hour - msg.time();
		this->buzzer = false;

		// Check if the snooze button was hit within
		// 49 minutes of the buzzer being turned on
		// for the first time
		if( this->timeRemaining <= ten_minutes )
		   {
		   // not enough time left for snoozing
		   this->snooz = false;
		   }
		else
		   {
		   // snooze for 10 minutes
		   this->snooz = true;
		   this->timeRemaining -= ten_minutes;
		   }

		holdIn( passive, Time::Zero );
	 	}	
	   } // end of if - else if statement

	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: Depending on the state of the model, this function
* can:
* - passivate the model
* - keep the buzzer on until 1 hour after it was first turned on
* - keep the buzzer off for 10 minutes (snooze)
********************************************************************/
Model &BuzDrv::internalFunction( const InternalMessage & )
{
	if( !this->snooz && !this->buzzer )
	   // not snoozing and the buzzer is off.  Either someone
	   // turned off the alarm using the <alarm_on> port
	   // or the buzzer reached its time limit of one
	   // hour after the alarm time.
	   this->passivate();
	else if( this->buzzer )
	   {
	   // buzzer is on.  Schedule an internal event to turn it off
	   // when the time limit is reached.
	   this->buzzer = false;
	   holdIn( active, this->timeRemaining );
	   }
	else
	   {
	   // buzzer is off and we are snoozing.  Schedule an internal
	   // event to turn the buzzer back on in 10 minutes.
	   this->buzzer = true;
	   holdIn( passive, ten_minutes );
	   }

	return *this;
}

/*******************************************************************
* Function Name: outputFunction
* Description: This routines outputs the value of the buzzer 
* variable to the <buzzer_on> port.
********************************************************************/
Model &BuzDrv::outputFunction( const InternalMessage &msg )
{
    sendOutput( msg.time(), this->buzzer_on, this->buzzer );

    return *this ;
}
