/*******************************************************************
*
*  DESCRIPTION: Display Driver Atomic Model
*
* TO DO clean up docs
*
*  This is the implementation of the Display Driver atomic model.
*  Its function is to select if the time of day or the alarm
*  time should be displayed.
*
*  The model has three inputs and one output defined as follows:
*
*  <time_of_day>: This input provides the time of day.
*
*  <alarm_time>: This input provides the alarm time.
*
*  <alarm_set>: This input is used by the driver to determine
*  which time to display.  If set to true, the alarm time is
*  sent to the output port so the user can see the alarm time
*  when manually setting it.  When <alarm_set> is false, the 
*  time of day is displayed.
*
*  <displayed_time>: This output provides either time of day or
*  alarm time. 
*
*  AUTHOR: Christian Jacques
*
*  EMAIL: chris.jacques@videotron.ca
*
*  DATE: 07 Oct 2001
*
*******************************************************************/

/** include files **/
#include "displayDriver.h"  // class DispDrv
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: DispDrv constructor
* Description: This routine constructs the DispDrv model
********************************************************************/
DispDrv::DispDrv( const string &name )
: Atomic( name )
, alarm_time( addInputPort( "alarm_time" ) )
, time_of_day( addInputPort( "time_of_day" ) )
, alarm_set( addInputPort( "alarm_set" ) )
, displayed_time( addOutputPort( "displayed_time" ) )
{
}

/*******************************************************************
* Function Name: DispDrv init function
* Description: This routine is invoked when simulation starts.
* This method simply initializes clears the alarm time and time of 
* day variables.
********************************************************************/
Model &DispDrv::initFunction()
{
	this->alarmTime = 0;
	this->timeOfDay = 0;
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: This method handles external events coming from any
* of the three input ports.
********************************************************************/
Model &DispDrv::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == this->alarm_set )
	   {
	   if( msg.value() == true)
		this->state( active );
	   else
		this->state( passive );

	   holdIn( this->state(), Time::Zero );
	   }
	else if( msg.port() == this->alarm_time )
	   {
	   this->alarmTime = msg.value();
	   if( this->state() == active )
		holdIn( active, Time::Zero );
	   }
	else // message came on the time_of_day port
	   {
	   this->timeOfDay = msg.value();
	   if( this->state() == passive )
		holdIn( passive, Time::Zero );
	   }
	
	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: The internal function simply keeps the model
* in its present state until the next external event comes.
********************************************************************/
Model &DispDrv::internalFunction( const InternalMessage & )
{
	holdIn( this->state(), Time::Inf );
	return *this;
}

/*******************************************************************
* Function Name: outputFunction
* Description: This routines outputs either the alarm time
* or the time of day to the <displayed_time> port.
********************************************************************/
Model &DispDrv::outputFunction( const InternalMessage &msg )
{
	if( this->state() == active)
	   sendOutput( msg.time(), displayed_time, this->alarmTime );
	else
	   sendOutput( msg.time(), displayed_time, this->timeOfDay );
	return *this ;
}
