/*******************************************************************
*
*  DESCRIPTION: Hours Register Atomic Model
*
*  This is the implementation of an hours register typically used
*  to create a clock.  The model has three inputs and one output 
*  defined as follows:
*
*  <time_set>: This input is used in conjunction with the <hours> input
*  to set the register to the desired hour {0, 1, ..., 23}.  This
*  signal is simply an enabler.  That is, the <hours> input value is 
*  disregarded unless the <time_set> input is active.
*
*  <hours>: Every time this input is received, the hour value is 
*  incremented by one as long as <time_set> is active.
*
*  <wrap_around>: This input is meant to be driven from a Minutes 
*  Register Atomic Model so the hour value can be incremented by one 
*  when the number of minutes wraps around from 59 to 60.  This
*  input is disregarded if <time_set> is active.
*
*  <time_hours>: This output represents the number of hours stored in 
*  the register.  Possible values are {0, 1, ..., 23}.
*  
*  AUTHOR: Christian Jacques
*
*  EMAIL: chris.jacques@videotron.ca
*
*  DATE: 28 September 2001
*
*******************************************************************/

/** include files **/
#include "hrRegister.h" // class HrReg
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: HrReg constructor
* Description: This routine constructs the HrReg model
********************************************************************/
HrReg::HrReg( const string &name )
: Atomic( name )
, time_set( addInputPort( "time_set" ) )
, hours( addInputPort( "hours" ) )
, wrap_around( addInputPort( "wrap_around" ) )
, time_hours( addOutputPort( "time_hours" ) )
{
}

/*******************************************************************
* Function Name: HrReg init function
* Description: This routine is invoked when simulation starts.  It 
* simply sets hour to 12.
********************************************************************/
Model &HrReg::initFunction()
{
	this->hour = 12;

	// Activate the output function so the hour is displayed
	// right away
	holdIn( passive, Time::Zero );

	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: This method handles external events coming from
* any of the three input ports.
********************************************************************/
Model &HrReg::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == time_set )
	   {

	   if( msg.value() == true ) 
		{
		// time_set was pressed.  Go active until 
		// an external event comes in.
		holdIn( active, Time::Inf );
		}
	   else {
		// time_set was released.  Go passive
		// until an external event comes in.
		passivate();
		}
	   }
	else if( msg.port() == hours )
	   {
	   // Only increment the hour if the model is in the 
	   // active phase.
	   if( state() == active )
		{
		if ( this->hour == 23 )
		   this->hour = 0;
		else
		  ++(this->hour);

		// Activate the output function immediately
		// so the new hour can be seen.
		holdIn( active, Time::Zero );
		}
	   }
	else if( msg.port() == wrap_around )
	   {
	   // Only increment the hour if the model is in the passive 
	   // state. 
	   if( state() == passive )
	   	{
		if ( this->hour == 23 )
		   this->hour = 0;
		else
		   ++(this->hour);

		// Activate the output function immediately
		// so the new hour can be seen.
		holdIn( passive, Time::Zero );
		}
	   }
	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: The internal function simply consists in keeping the
* model in the current state until an external event comes in. 
********************************************************************/
Model &HrReg::internalFunction( const InternalMessage & )
{
	holdIn ( state(), Time::Inf );
	return *this;
}

/*******************************************************************
* Function Name: outputFunction
* Description: This routine sens the value of the hour variable
* to the <time_hours> output port.
********************************************************************/
Model &HrReg::outputFunction( const InternalMessage &msg )
{
	sendOutput( msg.time(), time_hours, this->hour ) ;
	return *this ;
}
