/*******************************************************************
*
*  DESCRIPTION: Time Adder Atomic Model
*
*  This is the implementation of the time adder.  Its purpose
*  is to add minutes and hours to create a time of day.  The 
*  model has two inputs and one output defined as follows:
*
*  <time_hours>: This input provides the hour portion of the 
*  time.  It is an integer whose value should be between 0 and 
*  23.  However this rule is not enforced by the model.
*
*  <time_minutes>: This input provides the minute portion of the 
*  time.  It is an integer whose value should be between 0 and
*  59.  However, this rule is not enforced by the model.
*
*  <time_of_day>: This output provides the time of day by adding
*  the two inputs together.  It is a variable of type Time. 
*
*  AUTHOR: Christian Jacques
*
*  EMAIL: chris.jacques@videotron.ca
*
*  DATE: 05 October 2001
*
*******************************************************************/

/** include files **/
#include "timeAdder.h"  // class TimeAdder
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: TimeAdder constructor
* Description: This routine constructs the TimeAdder model
********************************************************************/
TimeAdder::TimeAdder( const string &name )
: Atomic( name )
, time_hours( addInputPort( "time_hours" ) )
, time_minutes( addInputPort( "time_minutes" ) )
, time_of_day( addOutputPort( "time_of_day" ) )
{
}

/*******************************************************************
* Function Name: TimeAdder init function
* Description: This routine is invoked when simulation starts.
* This method does nothing.
********************************************************************/
Model &TimeAdder::initFunction()
{
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: This method handles external events coming from any
* of the two input ports.
********************************************************************/
Model &TimeAdder::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == time_hours )
	   {
	   this->time.hours( msg.value() );
	   }
	else // the port is time_minutes
	   {
	   this->time.minutes( msg.value() );
	   }


	holdIn( passive, Time::Zero );

	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: The internal function simply passivates the model
********************************************************************/
Model &TimeAdder::internalFunction( const InternalMessage & )
{
	passivate ();
	return *this;
}

/*******************************************************************
* Function Name: outputFunction
* Description: This routines outputs the time to the <time_of_day>
* port. 
********************************************************************/
Model &TimeAdder::outputFunction( const InternalMessage &msg )
{
	// format the time for output.
	this->outTime = this->time.hours() * 100 + this->time.minutes();

	sendOutput( msg.time(), time_of_day, this->outTime );
	return *this ;
}
